<?php

declare(strict_types=1);

namespace Dedi\SyliusClicAndPayPlugin\Payum;

use Http\Message\MessageFactory;
use Payum\Core\Bridge\Spl\ArrayObject;
use Payum\Core\HttpClientInterface;
use Payum\Core\Reply\HttpPostRedirect;
use Payum\Core\Security\GenericTokenFactoryInterface;
use Sylius\Bundle\PayumBundle\Model\PaymentSecurityTokenInterface;
use Sylius\Component\Core\Model\PaymentInterface;

class ClicAndPayApi implements ClicAndPayParams
{
    const CODE = 'CLIC_AND_PAY';

    const ACCEPTED_STATUS = 'accepted';

    const CANCELLED_STATUS = 'cancelled';

    const REFUSE_STATUS = 'refuse';

    protected HttpClientInterface $client;

    protected MessageFactory $messageFactory;

    protected array $options = [];

    public function __construct(array $options, HttpClientInterface $client, MessageFactory $messageFactory)
    {
        $this->options = $options;
        $this->client = $client;
        $this->messageFactory = $messageFactory;
    }

    public function isAccepted(string $status): bool
    {
        return $status === ClicAndPayParams::AUTHORISED_STATUS;
    }

    public function isAbandoned(string $status): bool
    {
        return $status === ClicAndPayParams::ABANDONED_STATUS;
    }

    public function convertPayment(PaymentInterface $payment, PaymentSecurityTokenInterface $token, GenericTokenFactoryInterface $tokenFactory): array
    {
        $order = $payment->getOrder();
        $details = $payment->getDetails();
        $now = (new \DateTime())->setTimezone(new \DateTimeZone('UTC'));

        $details = ArrayObject::ensureArrayObject($details);
        $details[self::VADS_TRANS_DATE] = $now->format('YmdHis');
        $details[self::VADS_TRANS_ID] = $this->generateIdentifier($token->getHash());
        $details[self::VADS_AMOUT] = $order->getTotal();
        $details[self::VADS_ORDER_ID] = $order->getNumber();

        $details[self::VADS_RETURN_MODE] = 'GET';
        $details[self::VADS_URL_RETURN] = $token->getTargetUrl();
        $details[self::VADS_URL_SUCCESS] = $token->getTargetUrl();

        if ($this->options[self::IS_MULTI_PAYMENT]) {
            $details[self::VADS_PAYMENT_CONFIG] = sprintf(
                'MULTI:first=%d;count=%d;period=%s',
                round(((int) $this->options[self::MULTI_PAYMENT_FIRST_PERCENT] / 100) * $order->getTotal()),
                $this->options[self::MULTI_PAYMENT_COUNT],
                $this->options[self::MULTI_PAYMENT_PERIOD]
            );
        } else {
            $details[self::VADS_PAYMENT_CONFIG] = 'SINGLE';
        }

        if (!isset($details[self::VADS_URL_CHECK])) {
            $notifyToken = $tokenFactory->createNotifyToken($token->getGatewayName(), $payment);
            $details[self::VADS_URL_CHECK] = $notifyToken->getTargetUrl();
        }

        return (array) $details;
    }

    public function doPayment(array $details): void
    {
        if ($this->options['sandbox']) {
            unset($details[self::VADS_URL_CHECK]);
        }

        $details[self::VADS_ACTION_MODE] = $this->options[self::VADS_ACTION_MODE];
        $details[self::VADS_CTX_MODE] = $this->options['sandbox'] ? 'TEST' : 'PRODUCTION';
        $details[self::VADS_CURRENCY] = $this->options[self::VADS_CURRENCY];
        $details[self::VADS_PAGE_ACTION] = $this->options[self::VADS_PAGE_ACTION];
        $details[self::VADS_SITE_ID] = $this->options[self::VADS_SITE_ID];
        $details[self::VADS_VERSION] = $this->options[self::VADS_VERSION];

        $details[self::SIGNATURE] = $this->getSignature($details, $this->options['sandbox']);

        throw new HttpPostRedirect($this->getApiEndpoint(), $details);
    }

    protected function getSignature($details, $sandbox = false): string
    {
        $clientKey = $this->options[self::VADS_SITE_KEY];
        $signature = [];
        foreach (self::VADS_ORDER_TO_SIGNATURE as $key) {
            if ($key === self::VADS_URL_CHECK && $sandbox) {
                continue;
            }

            if (!isset($details[$key])) {
                throw new \RuntimeException('Clic and Pay : Need ' . $key . ' parameter to create signature.');
            }

            $signature[] = $details[$key];
        }

        $signature[] = $clientKey;

        return base64_encode(hash_hmac(self::HASH, implode('+', $signature), $clientKey, true));
    }

    protected function generateIdentifier(string $token): string
    {
        return array_reduce(str_split($token), static function ($token, $letter) {
            if (preg_match('/[a-zA-Z0-9]/', $letter) && strlen($token) < 6) {
                $token .= $letter;
            }

            return $token;
        }, '');
    }

    protected function getApiEndpoint(): string
    {
        return 'https://' . self::SERVERS[0] . '/' . self::URL_CLASSIC;
    }
}
